#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>


#define BUFF_SIZE           100

void* getServerSockAddr(struct sockaddr*);

int main(int argc, char** argv) {
    /* Usage: client <server> <port> */
    if(argc != 3) {
        fprintf(stderr, "Usage: %s <addr> <port>\n", argv[0]);
        return 1;
    }

    struct addrinfo hints = {};
    struct addrinfo* servAddrInfo;
    hints.ai_family = AF_UNSPEC;
    hints.ai_socktype = SOCK_STREAM;
    hints.ai_protocol = 0;

    int status = getaddrinfo(argv[1], argv[2], &hints, &servAddrInfo);
    if(status != 0) {
        fprintf(stderr, "getaddrinfo(): %s\n", gai_strerror(status));
    }

    int connSockfd;
    struct addrinfo* ptr;
    for(ptr = servAddrInfo; ptr != NULL; ptr = ptr->ai_next) {
        connSockfd = socket(servAddrInfo->ai_family,
                                servAddrInfo->ai_socktype,
                                servAddrInfo->ai_protocol);
        if(connSockfd == -1) {
            int tmp = errno;
            fprintf(stderr, "socket(): %s\n", strerror(tmp));
            continue;        
        }

        if(connect(connSockfd, 
                servAddrInfo->ai_addr, 
                servAddrInfo->ai_addrlen) == -1) {
            int tmp = errno;
            fprintf(stderr, "connect(): %s\n", strerror(tmp));
            close(connSockfd);
            continue;
        }

        break;
    }

    if(ptr == NULL) {
        freeaddrinfo(servAddrInfo);
        fprintf(stderr, "Failed to connect to all addr.\n");
        return 1;
    }

    char* servAddrStr[INET6_ADDRSTRLEN];
    inet_ntop(ptr->ai_family, 
              getServerSockAddr(ptr->ai_addr), 
              servAddrStr, 
              INET6_ADDRSTRLEN);
    fprintf(stdout, "Connected to %s\n", servAddrStr);
    freeaddrinfo(servAddrInfo);

    char buffer[BUFF_SIZE];
    int msgLen;
    msgLen = recv(connSockfd, buffer, BUFF_SIZE, 0);
    if(msgLen == -1) {
        int tmp = errno;
        fprintf(stderr, "recv(): %s\n", strerror(tmp));
        close(connSockfd);
        return tmp;
    }
    // printf("msgLen = %d\n", msgLen);
    buffer[msgLen] = '\0';
    // printf("strlen(buffer) = %d\n", strlen(buffer));
    
    close(connSockfd);
    fprintf(stdout, "server[%s:%s]: %s\n", argv[1], argv[2], buffer);

    return 0;
}

void* getServerSockAddr(struct sockaddr* server) {
    if(server->sa_family == AF_INET)
        return &(((struct sockaddr_in*)server)->sin_addr);
    else
        return &(((struct sockaddr_in6*)server)->sin6_addr);
}